#!/usr1/local/bin/python
'''
This library contians classes helpful in analyzing blast results.

This library depends on the following programs/packages;
	gi2fasta
	blastpgp
	formatdb
'''

import sys, os, cStringIO, shutil
import tempfile, copy

verbose = 0

#######################
#PSI-BLAST 
#This should be edited if blast program is not in path.
#######################
blast_program_dir = ''
blast_program = blast_program_dir + 'blastpgp'
formatdb_program = blast_program_dir + 'formatdb'

def set_blat_program_dir( dir ) :
	blast_program_dir = dir
	blast_program = blast_program_dir + 'blastpgp'
	formatdb_program = blast_program_dir + 'formatdb'

#######################

#######################
#Default DB
#This should be edited if nr is not a default db
#######################
default_db = ''
#######################

#Number of CPUs to use
#####################
number_of_cpus = 1
#####################

#####################

psiblast_end_of_result_marker = "  Database:"
psiblast_end_of_result_iteration = "Searching..................................................done"

psiblast_start = 'Results from round'
psiblast_start_m6 = 'QUERY '

hit_record_start = '>gi'
hsp_start = ' Score ='


class Domain :
	"""
	This class define a protein domain.
	Assumes the "gi2fasta" program in seals package.
	Also the indices for sequence should be given as python style!!
	"""
	def __init__(self, gi, start_index, end_index) :
		self.gi = gi
		self.start_index = int(start_index)
		self.end_index = int(end_index)
		self.sequence = ""
		self.annotation = ""

	def save_fasta( self, fn = "" ) :
		"""
		This function saves the sequence and information of the domain as fasta format.
		The default ID is now given as combination of self.gi and domain indices.
		"""
		if not fn :
			fn = '%s_%s..%s.fa'%( self.gi, start_index, end_index )
		
		fp = open( fn, 'w' )
		fp.write( self.get_fasta() )
		fp.close()

	def get_fasta( self ) :
		"""
		interface to get fasta format for the domain
		"""
		if self.sequence :
			pass
		else :
			self.set_domain_sequence()
			
		annotation = ">gir|%s_%s..%s|"
		if self.annotation :
			annotation += self.annotation
			if annotation[0] == '>' :
				pass
			else :
				annotation = '>' + annotation
		else :
			annotation += '>' + self.gi
		
		annotation += ' [pdr %d .. %d ]' % (self.start_index, self.end_index)
		
		return annotation + '\n' + self.sequence + '\n'


	def get_domain_sequence( self ) :
		"""
		interface to get domain sequence
		"""
		if self.sequence :
			return self.sequence

		gi, annot, seq = self.find_domain_sequence( self.gi )
		self.annotation = annotation
		if gi != self.gi :
			return ""

		self.sequence = seq[self.start_index, self.end_index]
		
		return self.sequence

	def set_domain_sequence( self ) :
		self.gi, self.annotation, self.sequence = self.find_domain_sequence( gi )

	def set_domain_sequence_from_blast_hsp( self, hsp ) :
		pass
		
	
	def find_domain_sequence( self, gi ) :
		fasta_fn = '%s.fa' % gi
		os.system( 'gi2fasta  %s > %s' %(gi, fasta_fn) )
		
		if os.path.exists( fasta_fn ) :
			pass
		else :
			return ""
	
		get_gi, anno, seq = self.read_fasta_file( fasta_fn )
		os.system( 'rm -f %s'%fasta_fn ) #cleanup
		return get_gi, anno, seq
		
	def read_fasta_file( self, fn ) :
		fp = open( fasta_fn )
		annotation = fp.readline()[1:-1]
		seq = ''.join( [ l.strip() for l in fp ] )

		gi = ""
		if annotation[:4] == '>gi|' :
			gi = annotation.split('|')[1]
		fp.close()

		return gi, annotation, seq


class DomainGroup :
	def __init__( self, group_name ) :
		self.domains = []
		self.group_name = group_name

	

	def add_domains_from_BlastMSA( self, blastmsa ) :
		for hit in blastmsa.hits :
			for hsp in hit.hsps :
	                        domain = Domain( hsp.gi, hsp.hit_start_index, hsp.hit_end_index )
                        	domain.annotation = blasthit.protein_name
                        	domain.sequence = hsp.hit.replace('-','')
                        	
                        	self.domains.append( domain )

		
	def add_domains_from_BlastHit( self, blasthit ) :
		for hsp in blasthit.hsps :
			domain = Domain( hsp.gi, hsp.hit_start_index, hsp.hit_end_index )
			domain.annotation = blasthit.protein_name
			domain.sequence = hsp.hit.replace('-','')
			
			self.domains.append( domain )
		
	

class BlastHit :
	"""
	This class can contains multiple HSPs for a hit.
	GI is used as a unique identifier for a hit.
	"""

	def __init__(self) :
		self.gi = 0
		self.protein_name = ""
		self.hsps = []
		self.length=0
		
		self.hit_start = hit_record_start #'>gi'
		self.hit_info_end = '          Length ='
		self.index_offset = 0 #for gir correction

		self.hsp_length_cutoff = 30 #length cutoff for hsp

	def add_hit_as_hsp( self, hit ) :
		for hsp in hit.hsps :
			self.hsps.append( hsp )

	def divide_hsp( self, hsp, positions ) :
		'''
		Divide given hsp into fragments according to given starting positions (in query column positions)
		and return all of them.
		positions should be a sorted integer vector.
		'''

		if not positions :
			return None

		new_hsps = []
		

		'''
		#by mistake this block is written .. :(
		#first hsp
		if hsp.check_within( positions[0] )==0 :
			new_hsp = copy.deepcopy( hsp )
			new_hsp.adjust_query_region( new_hsp.query_start_index, positions[0] )
			new_hsps.append( new_hsps )
		else :
			return new_hsps
		'''

		if verbose :
			print '#dividing HSP positions:', positions
			print 'hsp start_index:', hsp.query_start_index, 'query_end_inde', hsp.query_end_index

		#before current cutting position
		if hsp.check_within( positions[0] ) == 0:
			new_hsp= copy.deepcopy( hsp )
			if new_hsp.adjust_query_region( new_hsp.query_start_index, positions[0] ) :
				new_hsps.append( new_hsp )


		elif hsp.check_within(positions[0] ) == -1 :
			pass
		else :
			return new_hsps
		
		for i, pos in enumerate( positions[0:-1] ):

			if verbose :
				print 'Trying to break at position', pos, '...'

			if hsp.check_within( pos ) == 0: #check if the given pos is within the hsp range

				new_hsp = copy.deepcopy( hsp )
				if hsp.check_within( positions[i+1] ) == 0 :
					if new_hsp.adjust_query_region( pos, positions[i+1] ) :
						new_hsps.append( new_hsp )
				elif hsp.check_within( positions[i+1] ) == -1 :
					pass
				else :
					if new_hsp.adjust_query_region( pos, new_hsp.query_end_index ) :
						new_hsps.append( new_hsp )
					return new_hsps

			elif hsp.check_within( pos ) == -1 :
				pass 
			else :
				return new_hsps

		else :
			if hsp.check_within( positions[-1] ) == 0 :
			
				new_hsp = copy.deepcopy( hsp )
				if new_hsp.adjust_query_region( positions[-1], new_hsp.query_end_index ) :
					new_hsps.append( new_hsp )
				return new_hsps
			else :
				return new_hsps

		

	def purge_overlapping_hsps( self, indices, pssm ) :
		'''
		purge hsps by selecting good regions in the hsps
		'''
		if not indices :
			return 

		if verbose :
			print '###','Purging HIT', self.gi, '###'

		for i, hsp in enumerate(self.hsps[:]) :
			#select correct region among the segments
			#first, divide query string into fragments using given indices
			#second, calculate the score for each fragments.
			new_hsps = self.divide_hsp( hsp, indices )
			if verbose :
				print 'HSP #%d are divided into'%i, new_hsps

			if len(new_hsps)<=1 :
				if verbose :
					print 'purging cannot be done', 'new_hsps=', new_hsps
				continue

			if verbose :
				for j,h in enumerate( new_hsps ) :
					print 'new_hsps:',j, h.query, h.hit

			pssm_scores = [pssm.cal_pssm_score(h) for h in new_hsps]
			scores_hsps = zip(pssm_scores, new_hsps)
			scores_hsps.sort()
			if verbose :
				print scores_hsps
			self.hsps[i] = scores_hsps[-1][1]
			
			if verbose :
				print 'score-hsp pairs:', scores_hsps
				print '##selected HSP:', self.hsps[i].query_start_index, self.hsps[i].query, self.hsps[i].query_end_index, '\n'
	
		#check the correct fragment mapping
		#by overlapping of regions

		for i, hsp in enumerate( self.hsps ) :
			for hsp2 in self.hsps[i+1:] :
				if hsp.is_overlapping( hsp2 ) :
					if verbose :
						print "Warning HSP selection has problem!"
					#removing all hsps
					self.hsps = []
					########################
					#this hit instance should also be removed
					########################
		else :
			if verbose :
				print 'HSP checking is done!'
		
		
		"""
		Blocked out the removing small fragment part
		#remove small fragments
		#maybe not necessary
		for hsp in self.hsps[:] :
			if len(hsps.hit) <= self.hsp_length_cutoff :
				self.hsps.remove( hsp )
		"""

	def remove_overlapped_hit_region( self, hit ) :
		"""
		Remove all the common hit matching regions.
		"""

		for hsp2 in hit.hsps :
			for hsp in self.hsps[:] : #to use slice to avoid problem in removing elements.
				if verbose :
					print "#### Hit", self.gi, "####"
					print  "hit1:", hsp.hit_start_index, hsp.hit_end_index
					print  "hit2:", hsp2.hit_start_index, hsp2.hit_end_index

				self.remove_overlapped_hit_region_hsp( hsp, hsp2 )
				if hsp in self.hsps and len(hsp.hit) <= self.hsp_length_cutoff :
					self.hsps.remove( hsp )


	def remove_overlapped_hit_region_hsp(self, hsp1, hsp2 ) :
		"""
		Remove all the common area between two HSPs.
		This function defined in BlastHit class not in BlastHSP class
		because the manipulation of hsps is required in Hit level.

		Note that this is still not tested and this functions carries on "Hit" side 
		overlapping checking and removing.
		The one that we need is query (for the current purging test).
		"""

		#These value will be set
		new_start1 = 0
		new_end1 = 0

		start1 = hsp1.hit_start_index
		end1 = hsp1.hit_end_index
		
		start2 = hsp2.hit_start_index
		end2 = hsp2.hit_end_index

		if end1 <= start2 or end2 <= start1 :
			if verbose :
				print  "No overlap between the two hsps"
			return 0

		if start2 <= start1 and  start1 < end2 :
			if end2 < end1 :
				if verbose :
					print  "Overlap in N-terminal of hit"
				new_start1 = end2
				new_end1 = end1
			
				return hsp1.adjust_hit_region( new_start1, new_end1 )

			else :
				if verbose :
					print   "Hit2 contains hit"
				self.hsps.remove( hsp1 )
				return end1-start1

		elif start1 <= start2 and start2 < end1 :
			if end1 <= end2 :
				if verbose :
					print   "Overlap in C-terminal of hit"
				new_start1 = start1
				new_end1 = start2

				return hsp1.adjust_hit_region( new_start1, new_end1 )

			#This breaks hsp1 into two halves
			elif end2 < end1 :
				if verbose :
					print   "Dividing into two"

				first_half = hsp1
				second_half = copy.deepcopy( hsp1 )
				self.hsps.append(second_half)

				new_start1 = start1
				new_end1 = start2

				first_half.adjust_hit_region( new_start1, new_end1 )

				new_start1 = end2
				new_end1 = end1

				second_half.adjust_hit_region( new_start1, new_end1 )
		
		else :
			print >>sys.stderr, "Error! This message should not be seen!! Function", "remove_overlapped_hit_region_hsp"

	def find_inserted_domains( self ) :
		'''
		Find and return the inserted domain gi and the region
		Finding algorithm is 
		1. proteins very close to the query (identity > 95)
		2. select longest protein
		3. find regions that is not mapped to the query domains
		
		This function assumes the ordering of the domains are conserved between close homologs.
		'''

		removing_threshold = 5 #if the overlapping is below this just removing them
		domain_threshold = 30
		
		
		hit_mappings = [ hsp.get_hit_region() for hsp in self.hsps ]
		query_mappings = [ hsp.get_query_region() for hsp in self.hsps ]
	
		mappings = zip(query_mappings, hit_mappings)
		mappings.sort()
		
		#q1 = [i, j] 
		#where i and j are start index and end index of query
		#similarity h1 are for hit.
		#The region indices are defined like python style.
		for (q1, h1), (q2, h2) in zip( mappings, mappings[1:] ) :
			overlap = q2[0] - q1[1]
			if overlap < removing_threshold :
				q2[0] += overlap
				q1[1] -= overalp
				
				h2[0] += overlap
				h1[1] -= overalp
		
			hit_overlap = h2[0] - h1[1] 
			if hit_overlap > domain_threshold :
				pass
				
		
	def determine_break_points( self ) :
		'''
		Find and return the breaking points.
		This function should be used only to the close hits.
		incomplete
		'''
		break_points = []
		if len( self.hsps ) < 2 :
			return break_points

		hsps_regions = []
		for hsp in self.hsps :
			hsps_regions.append( hsps.get_query_region() )
			

		pass
		
			
	def get_query_overlap_regions( self ) :
		'''
		helper function to get the overlapping region
		'''
		overlaps = []
		if len( self.hsps ) < 2 :
			return overlaps
		
		pass
		
		
	def add_hsp( self, hsp ) :
		self.number_of_hsps += 1
		self.hsps.append( hsp )
	
	def combine_hsp( self ) :
		"""
		This is a testing function to solve the profile corruption
		"""
		pass

	def get_aligned_hit_sequence( self, query_length ) :
		"""
		This function does not alter any results from blast, but makes aligned input
		"""
		return ''.join( [ '%-20s %s'%(self.gi[:20], hsp.get_aligned_hit_sequence(query_length)) for hsp in self.hsps ] )
		

	def get_aligned_hit_sequence_combined( self, query_length ) :
		"""
		This function makes aligned input.
		Note that this function combines all hsps by putting first occurance of residue
		and if all columns are gaps then put gap symbol ('-').
		"""
		sequences = []
		for hsp in self.hsps :
			sequences.append( hsp.get_aligned_hit_sequence( query_length ) )
			
		
		if len(sequences ) == 0 :
			return ""
		elif len(sequences) == 1 :
			return "%-20s %s" %( self.gi[:20], sequences[0] )
		
		str_fp = cStringIO.StringIO()
		for j in xrange( len( sequences[0] )-1 ) : #position j and it is adjust to get rid of the last new line
			for i in sequences : #sequence i
				if i[j].isalpha() :
					str_fp.write( i[j] )
					break
			else :
				str_fp.write( '-' )
		str_fp.write( '\n' )
		return "%-20s %s" %( self.gi[:20], str_fp.getvalue() )
	

	def remove_overlapping_hsps( self ) :
		'''
		This is a testing function to solve the profile corruption.
		The idea is removing possibly contaminated HSPs from the hit list.
		purging idea #3
		'''
		hsps_toberemoved = []
		for hsp1 in self.hsps :
			for hsp2 in self.hsps :
				if hsp1 == hsp2 :
					continue
				
				if hsp1.is_overlapping( hsp2 ) :
					hsps_toberemoved.append( hsp1 )
					break

		for hsp in hsps_toberemoved :
			self.hsps.remove( hsp )

	def remove_overlapping_regions_in_hsps( self ) :
		'''
		This is a testing function to solve the profile corruption.
		The idea behind this is removing overlapping regions that possibly contaminates
		the MSA built by blast.

		purging idea #3-2
		'''

		new_hsps = copy.deepcopy( self.hsps )
		for hsp1 in new_hsps :
			for hsp2 in self.hsps :
				if hsp1.is_overlapping( hsp2 ) :
					hsp1.remove_overlapping_region( hsps2 )
		
		self.hsps = new_hsps


	def to_string( self ) :
		s = "gi: %s\n"% self.gi
		for hsp in self.hsps :
			s += hsp.to_string()
		return s
		
		
	def parse( self, fp ) :
		'''
		Returns 0 for nomal end 
		Returns -2 for nomal psiblast/blast end 
		Returns -1 for errorneous parsing
		returns 1 for successful parsing
		'''
		hit_line = fp.readline()

		if verbose :
			print 'first hit line check:', hit_line

		while hit_line[:len(self.hit_start)] != self.hit_start :
			if hit_line[:len( psiblast_end_of_result_marker )] == psiblast_end_of_result_marker  or hit_line[:len(psiblast_start)] == psiblast_start :
				if verbose :
					print "End of Blast Result reached!", hit_line 

				fp.seek(-len(hit_line), 1 ) #rewind one line back for the parameters for blast run
				return -2

			hit_line = fp.readline()
			if verbose :
				print 'subsequent hit line check:', hit_line

			if not hit_line  :
				return -1
		if verbose :
			print 'finally supposed to be correct hit line:', hit_line

		#getting gi from a line
		#>gi|74188315|dbj|BAE25816.1| unnamed protein product [Mus musculus]
		gi = hit_line.split('|')[1] #representative hit gi is the first one
		protein_name = hit_line[1:-1]

		#consume all other lines
		#stops at 
		#          Length = 566
		info_line = fp.readline()
		while info_line and info_line[:len(self.hit_info_end)] != self.hit_info_end :
			info_line = fp.readline()
		
		length = int(info_line.split()[-1])
		fp.readline() #consume one empty line

		#adjustment for gir: gi with range
		#gir consists of gi_start..end
		#e.g. 74188315_1..120
		index_offset = 0
		if gi.find('_') == -1 :
			pass
		else :
			gi, r = gi.split('_')
			index_offset = int(r.split('..')[0])

		self.gi = gi
		self.length = length
		self.index_offset = index_offset

		hit_fp = cStringIO.StringIO()
		while True :
			line = fp.readline()
			if not line :
				break
			if line[:len(self.hit_start)] == self.hit_start :
				fp.seek( -len(line), 1 )
				break

			hit_fp.write(line)

		hit_fp.seek(0) #rewinde hit_fp

		hsp = BlastHSP( offset=self.index_offset )
		parse_flag = hsp.parse(hit_fp)
		if verbose :
			print '@ after the first BlastHSP.parse(), parse_flag:', parse_flag
			
		while parse_flag == 1 :
			self.hsps.append(hsp)

			hsp = BlastHSP( offset=self.index_offset )
			parse_flag = hsp.parse( hit_fp )

			if verbose :
				print '@ after the subsequent BlastHSP.parse(), parse_flag:', parse_flag
		if verbose :
			print '@ End of BlastHit.parse(), parse_flag:', parse_flag
	
		if parse_flag == 0 :
			return 1 #just HSP end!!
	
		return parse_flag

	def write_hit_fasta( self, fp ) :
		for hsp in self.hsps :
			hsp.write_hit_fasta( fp, self.gi )
		

def overlapping_checker ( s1, e1, s2, e2 ) :
	"""
	Check overlapping for the given 4 numbers (2 pairs of start and end)
	start1, end1, start2, end2
	"""

	#no overlapping case
	if s1 < s2 and e1 <= s2 :
		return 0
	if s2 < s1 and e2 <= s1 :
		return 0

	#inclusion case 
	if s1 <= s2 and e2 <= e1 :
		return 1
	if s2 <= s1 and e1 <= e2 :
		return 1

	#partial overlap
	if s2 < e1 and e1 <= e2 :
		return 1

	if s1 < e2 and e2 <= e1 :
		return 1

	#This is non reachable case
	print >>sys.stderr, "Error! This message should not be seen!!"
	print >>sys.stderr, "overlapping checker", s1, e1, s2, e2
	return 1


class BlastHSP :

	def __init__ ( self, offset=0 ) :
		self.query_start_index = 0
		self.query_end_index = 0
		self.hit_start_index = 0
		self.hit_end_index = 0
		
		self.query = None
		self.hit = None

		self.evalue = 0.0
		self.bitscore = 0.0
		self.score = 0
		self.identity = 0.0
		self.positive = 0.0
	
		self.hsp_start = hsp_start #' Score ='
		self.query_line_start = 'Query:'
		self.hit_line_start = 'Sbjct:'
		self.modified = 0

		self.overlapping_mark = '='
		self.orignal_hit = ''
		self.index_offset = offset #hit index offset


	def check_within( self, pos, query=True ) :
		'''
		Checks if the given position is within Query (or Hit) range.
		check_within( pos ) returns -1 if pos is before the range.
		0 if pos is within the range.
		1 if pos is after the range.
		'''
		start = self.query_start_index
		end = self.query_end_index

		if not query :
			start = self.hit_start_index
			end = self.hit_start_index
		
		if pos <= start :
			return -1
		elif start < pos < end :
			return 0
		else : #elif end <= pos : same as this
			return 1


	def is_overlapping( self, hsp ) :
		'''
		Check for the query side overlapping.
		'''
		return overlapping_checker( self.query_start_index, self.query_end_index, hsp.query_start_index, hsp.query_end_index )

	def mark_overlapping_hit_regions( self, query_start, query_end ) :
		'''
		Mark overlapped region.
		The mark should be removed to make the overlap removed version can be used for building profiles.
		Also note that this function makes the hsp hit region information not reliable.

		It is very important to know that the hit region is being marked, 
		but the start and end information should
		be given by query residue count!!

		Original hit alignment will be saved into the original_hit variable.
		'''

		if self.modified == 0 :
			self.original_hit = self.hit
			self.modified = 1
		
		hit_start = self.find_corresponding_hit_index( query_start )
		hit_end = self.find_corresponding_hit_index( query_end )

		self.hit = self.hit[:hit_start] + self.overlapping_mark * (query_end-query_start) + self.hit[query_end:]
		

	def remove_overlapping_mark( self ) :
		'''
		Remove the overlapping marks and replace them into gap marks '-'.
		'''
		fp = cStringIO.StringIO()
		for a in self.hit :
			if a == self.overlapping_mark :
				fp.write('-')
			else :
				fp.write( a )
	
		self.hit = fp.getvalue()

	def find_corresponding_hit_index( self, query_index ) :
		'''
		Find hit index (real index for hit string) query start point.
		Since query string may have a gap for the exact point equivalent for hit_index
		the function will return right after the non-gap query index and the corresponding hit_index
		'''
		qi = self.query_start_index -1 #query index is real residue index
		#hi = 0 #hit is string index so it counts all positions including gaps

		for i,a in enumerate( self.query ) :
			if a.isalpha()  :
				qi += 1

			if qi >= query_index :
				return i

		return len(self.hit)

	def convert_position_to_real_index( self, aligned_seq, start_index, pos ) :
		'''
		Convert sequence position (for hit or query) into real index according to aligned_seq.
		The aligned_seq should be hit string or query string.
		if conversion failes then this function returns length of aligned_seq
		'''
		current_pos = start_index -1
		real_index = -1
		for i,a in enumerate(aligned_seq) :
			if a.isalpha() :
				current_pos += 1
				
			if pos == current_pos :
				real_index = i
				break
		else :
			real_index = len( aligned_seq )
			
		return real_index

	def convert_real_index_to_position( self, aligned_seq, start_index, real_index ) :
		'''
		Convert real index into position (for hit or query) according to given aligned_seq and start_index.
		The aligned_seq should be self.hit or self.query.
		If conversion fails then this function returns end position. (note that the end position is in python slice style)
		'''
		current_pos = start_index - 1
		for i, a in enumerate( aligned_seq ) :
			if a.isalpha() :
				current_pos += 1
			
			if i == real_index :
				return current_pos
		else :
			current_pos += 1
			return current_pos


	def find_corresponding_query_start_index( self, hit_index ) :
		'''
		Find query start point.
		Since query string may have a gap for the exact point equivalent for hit_index
		the function will return right after the non-gap query index and the corresponding hit_index
		'''
		qi = self.query_start_index -1
		hi = self.hit_start_index -1
		qi_last = qi
		hi_last = hi
		for i,(a,b) in enumerate(zip(self.query, self.hit)) :
			querygap = 1
			if a.isalpha()  :
				qi += 1
				querygap = 0

			if b.isalpha() :
				hi += 1

			if hi >= hit_index and querygap == 0 :
				qi_last = qi
				hi_last = hi
				break

		return qi_last, hi_last


	def find_corresponding_hit_start_index( self, query_index ) :
		'''
		Find query start point.
		Since query string may have a gap for the exact point equivalent for hit_index
		the function will return right after the non-gap query index and the corresponding hit_index
		'''
		qi = self.query_start_index -1
		hi = self.hit_start_index -1
		qi_last = qi
		hi_last = hi
		for i,(a,b) in enumerate(zip(self.query, self.hit)) :
			hitgap = 1
			if a.isalpha()  :
				qi += 1

			if b.isalpha() :
				hitgap = 0
				hi += 1

			if qi >= query_index and hitgap == 0 :
				qi_last = qi
				hi_last = hi
				break

		return qi_last, hi_last




	def find_corresponding_query_end_index( self, hit_index ) :
		'''
		Find query end point.
		Since query string may have a gap for the exact point equivalent for hit_index
		the function will return right before the non-gap query index and the corresponding hit_index
		'''
		qi = self.query_start_index -1
		hi = self.hit_start_index -1
		qi_last = qi
		hi_last = hi
		for i,(a,b) in enumerate(zip(self.query, self.hit)) :
			querygap = 1
			if a.isalpha()  :
				qi += 1
				querygap = 0

			if b.isalpha() :
				hi += 1

			if hi <= hit_index and querygap == 0 :
				qi_last = qi
				hi_last = hi
				
			if hi >= hit_index :
				break
		else :
			#since the end should be out of the region.
			return qi_last+1, hi_last+1

		return qi_last, hi_last


	def find_corresponding_hit_end_index( self, query_index ) :
		'''
		Find hit end point.
		Since hit string may have a gap for the exact point equivalent for query_index
		the function will return right before the non-gap query index and the corresponding hit_index
		'''
		qi = self.query_start_index -1
		hi = self.hit_start_index -1
		qi_last = qi
		hi_last = hi
		for i,(a,b) in enumerate(zip(self.query, self.hit)) :
			hitgap = 1
			if a.isalpha()  :
				qi += 1

			if b.isalpha() :
				hi += 1
				hitgap = 0

			if qi <= query_index and hitgap == 0 :
				qi_last = qi
				hi_last = hi
				
			if qi >= query_index :
				break
		else : 
			return qi_last+1, hi_last+1

		return qi_last, hi_last


	def set_trimed_query( self, start_index, end_index ) :
		temp_fp = cStringIO.StringIO()

		#start to write query_string
		qi = self.query_start_index -1
		for a in self.query :
			if a.isalpha() :
				qi += 1

			if qi >= end_index :
				break

			if start_index <= qi and qi < end_index :
				temp_fp.write(a)

		temp_fp.write("") #reset buffer

		self.query_start_index = start_index
		self.query_end_index = end_index
		self.query = temp_fp.getvalue()

	
	def set_trimed_hit( self, start_index, end_index ) :
		temp_fp = cStringIO.StringIO()

		#start to write hit_string
		qi = self.hit_start_index -1
		for a in self.hit :
			if a.isalpha() :
				qi += 1

			if qi >= end_index :
				break

			if start_index <= qi and qi < end_index :
				temp_fp.write(a)

		temp_fp.write("") #reset buffer

		self.hit_start_index = start_index
		self.hit_end_index = end_index
		self.hit = temp_fp.getvalue()

	

	def adjust_hit_region( self, new_hit_start, new_hit_end ) :
		'''
		change current query and hit region by reducing into new_hit_start and new_hit_end
		'''
		if new_hit_start >= self.hit_end_index or new_hit_end <= self.hit_start_index :
			print >>sys.stderr, "Warning! hit adjust failed!", self.hit_start_index, self.hit_end_index, "to", new_hit_start, new_hit_end
			return 0

		#if the new region is OK.
		self.modified=1

		#getting query and hit start indices
		new_qsi, new_hsi = self.find_corresponding_query_start_index( new_hit_start )
		new_qei, new_hei = self.find_corresponding_query_end_index( new_hit_end )
		
		si = self.convert_position_to_real_index( self.query, self.query_start_index, new_qsi )
		ei = self.convert_position_to_real_index( self.query, self.query_start_index, new_qei )
		new_query = self.query[ si : ei ]

		if verbose :
			print  'old query:', self.query_start_index, self.query, new_qsi, si, new_query
	
		si = self.convert_position_to_real_index( self.hit, self.hit_start_index, new_hsi )
		ei = self.convert_position_to_real_index( self.hit, self.hit_start_index, new_hei )
		new_hit = self.hit[ si : ei ]
		
	
		if verbose :
			print  'old hit:', self.hit_start_index, self.hit, new_hsi, si, new_hit

		self.query = new_query
		self.hit = new_hit

		self.query_start_index = new_qsi
		self.query_end_index = new_qei
		self.hit_start_index = new_hsi
		self.hit_end_index = new_hei


		
	def adjust_query_region( self, new_query_start, new_query_end ) :
		'''
		change current query and hit region by reducing into new_hit_start and new_hit_end
		'''
		if verbose :
			print 'adjust query region from ', self.query_start_index, '-', self.query_end_index, 'to', new_query_start, '-', new_query_end

		if new_query_start >= self.query_end_index or new_query_end <= self.query_start_index :
			print "Warning! hit adjust failed!", self.query_start_index, self.query_end_index, "to", new_query_start, new_query_end
			return 0

		#if the new region is OK.
		self.modified=1

		#getting query and hit start indices
		new_qsi, new_hsi = self.find_corresponding_hit_start_index( new_query_start )
		new_qei, new_hei = self.find_corresponding_hit_end_index( new_query_end )
		
		si1 = self.convert_position_to_real_index( self.query, self.query_start_index, new_qsi )
		ei1 = self.convert_position_to_real_index( self.query, self.query_start_index, new_qei )
		new_query = self.query[ si1 : ei1 ]
		
		if not new_query :
			if verbose: print 'Warning! Query adjustment made null string', self.query, self.query_start_index, '(%s)'% new_qsi, self.query_end_index, '(%s)'%new_qei
			return 0


		if verbose :
			print  'old query:', self.query_start_index, self.query, self.query_end_index, new_qsi, '(',si1,')', new_query, new_qei, '(', ei1, ')'

		si2 = self.convert_position_to_real_index( self.hit, self.hit_start_index, new_hsi )
		ei2 = self.convert_position_to_real_index( self.hit, self.hit_start_index, new_hei )
		new_hit = self.hit[ si2 : ei2 ]
		if not new_hit :
			if verbose: print 'Warning! Hit adjustment made null string', self.hit, self.hit_start_index, '(%s)'% new_hsi, self.hit_end_index, '(%s)'%new_hei
			return 0

		if verbose :
			print  'old hit:', self.hit_start_index, self.hit, self.hit_end_index, new_hsi, '(', si2, ')',  new_hit, new_hei, '(',ei2 , ')'

		if si1 != si2 or ei1 != ei2 :
			print >>sys.stderr, 'Warning! si1 and si2, ei1 and ei2 should be same! si1, si2, ei1, ei2:', si1, si2, ei1, ei2
			print >>sys.stderr, 'Warning1', 'new_query:', new_query
			print >>sys.stderr, 'Warning2', 'new_hit  :', new_hit

		self.query = new_query
		self.hit = new_hit

		self.query_start_index = new_qsi
		self.query_end_index = new_qei
		self.hit_start_index = new_hsi
		self.hit_end_index = new_hei

		return 1
	

	def write_hit_fasta( self, fp, gi, annot="" ) :

		hit = self.hit.replace('u', 'x') #make cleaning
		hit = hit.replace('U', 'X')
		hit = hit.replace('-', '')

		str = ">gir|%s_%d..%d|%s\n%s\n" % (gi, self.hit_start_index, self.hit_end_index, annot, hit )
		fp.write( str )
		
	def get_query_region( self ) :
		return [ self.query_start_index, self.query_end_index ]

	def get_hit_region( self ) :
		return [self.hit_start_index, self.hit_end_index ]

	def to_string( self ) :
		s = "****\nbitscore: %f evalue: %e\n" %( self.bitscore, self.evalue )
		s += "Query start:%d end:%d\n" % (self.query_start_index, self.query_end_index)
		s += "Sbjct start:%d end:%d\n" % (self.hit_start_index, self.hit_end_index)
		s += "%s\n%s\n" % (self.query, self.hit)
		s += "****\n"
		return s


	def get_aligned_hit_sequence( self, query_length ) :
		aln_fp = cStringIO.StringIO()

		#Pre alingment insertion
		pre_aln_string = '-'* self.query_start_index
		aln_fp.write( pre_aln_string )

		#alignment main part
		for a, b in zip( self.query, self.hit ) :
			if a.isalpha() :
				aln_fp.write(b)

		#post alignment insertion
		post_aln_length = query_length - aln_fp.tell()
		post_aln_string = '-'*post_aln_length
		aln_fp.write( post_aln_string )
		aln_fp.write('\n')
		
		return aln_fp.getvalue()

	
	def parse( self, fp ) :
		'''
		Returns 0 for normal end 
		Returns -2 for PSI-BLAST end
		Returns 1 for successful parsing of BlastHSP
		Returns -1 for non-normal end or errorneous parsing.
		'''
		line = fp.readline()

		if verbose :
			print '@@ BlastHSP.parse() first line:', line

		while line[:len(self.hsp_start)] != self.hsp_start :

			if line[:len( psiblast_end_of_result_marker )] == psiblast_end_of_result_marker or line[:len(psiblast_start)] == psiblast_start :
				fp.seek(-len(line), 1 ) #rewind one line back for the parameters for blast run
				if verbose :
					print "End of Blast Result reached! @HSP paring:", line 

				return -2


			line = fp.readline()

			if verbose :
				print '@@ BlastHSP.parse() skipping lines:', line
			
			if not line :
				return 0

		if verbose :
			print '@@ BlastHSP.parse() after initial line check', line
			
		score_line1 = line.split()
		score_line2 = fp.readline().split()

		self.bitscore = float( score_line1[2] )
		if score_line1[7][0] == 'e' :
			score_line1[7] = '1' + score_line1[7]
		self.evalue = float(score_line1[7][:-1])
		n1, n2 = score_line2[2].split('/')
		self.identity = float( int(n1)*1.0/int(n2) )
		n1, n2 = score_line2[2].split('/')
		self.positive = float( int(n1)*1.0/int(n2) )

		#push aray one line
		line = fp.readline() #white line
		
		#query line
		query_line = fp.readline()
		line = query_line.split()

		if verbose :
			print '@@ BlastHSP.parse() query_line:', query_line

		if line and line[0] == self.query_line_start :
			pass
		else :	
			print >>sys.stderr, "Error in parsing Query line of HSP", query_line
			return -1

		#save query and hit info temporarily
		query_start_index = int(line[1]) - 1
		query_end_index = int(line[3])
		query = line[2]

		#positive line
		line = fp.readline()
		
		#hit line
		hit_line = fp.readline()
		line = hit_line.split()

		if verbose :
			print '@@ BlastHSP.parse() hit_line:', hit_line

		if line[0] == self.hit_line_start :
			pass
		else  :
			print >>sys.stderr, "Error in parsing Query line of HSP", query_line
			return -1

		hit_start_index = int(line[1]) - 1
		hit_end_index = int(line[3])
		hit = line[2]

		fp.readline() #consume one line

		while True :
			#query line
			query_line = fp.readline()

			if verbose :
				print '@@ BlastHSP.parse() in while query_line:', query_line
			if not query_line :
				break
			if not query_line[:len(self.query_line_start)] == self.query_line_start :
				break
				
			line = query_line.split()

			query_end_index = int(line[3])
			query += line[2]
			
			#positive line
			line = fp.readline()
			
			#hit line
			hit_line = fp.readline()
			if verbose :
				print '@@ BlastHSP.parse() in while hit_line:', hit_line

			if not hit_line :
				return -1
			if not hit_line[:len(self.hit_line_start)] == self.hit_line_start :
				return -1

			line = hit_line.split()

			hit_end_index = int(line[3])
			hit += line[2]

			fp.readline() #consume one line

		self.query = query
		self.hit = hit
		self.query_start_index = query_start_index 
		self.hit_start_index = hit_start_index + self.index_offset
		self.query_end_index = query_end_index
		self.hit_end_index = hit_end_index + self.index_offset
		
		return 1
			
	def realign( self ) :
		"""
		Test for realign idea
		"""
		pass

	def select_best_hsp( self ) :
		"""
		This function is for removing suboptimal HSPs and selecting the best HSP only.
		For test purpose 
		"""
		pass
	
class BlastMSA :
	def __init__( self, seq_fn, blast_result_fn="" ) :
		self.hits = []
		self.query_fn = seq_fn
		self.annotation = ""
		self.query_seq = ""
		self.blast_result_fn = blast_result_fn
		self.annotation, self.query_seq = self.get_query( seq_fn )
		self.effective_length_of_the_database = 0
		self.dummy_db = ''

		self.gi2hit = {}

		self.pssm = None #position specific score matrix
		self.parsing_status = None

		if blast_result_fn :
			self.parse_from_file( blast_result_fn )


	def __del__( self ) :
		if self.dummy_db :
			self.remove_temporary_dummy_db()


	def build_pssm( self, msa ="" ) :
		if not msa :
			msa = self

		if self.pssm :
			return 

		if self.dummy_db :
			pass
		else :
			self.build_temporary_dummy_db()

		pssm = ScoreMat( len(msa.query_seq) )#runner.getPSSM()
		pssm.build_pssm( msa )
		self.pssm = pssm
		self.remove_temporary_dummy_db() #cleanup temporary file

		return pssm


	"""
	#to fix some problem in prodata it is changed into 
	#using tempfile.mkstemp()
	def build_profile( self ) :
		'''
		This function builds a profile for the sequence using blastpgp.
		BlastMSA object for the blast hit result will be returned.
		'''

		print "Building profile for ", self.query_fn, "..."

		if self.dummy_db :
			self.build_temporary_dummy_db() 
		
		msa_fp = tempfile.NamedTemporaryFile()
		self.build_psiblast_alignment_input( msa_fp )
		msa_fp.flush()
			
		runner = PsiblastRunner( self.query_fn )
		profile_fn = runner.run_blast_for_profile( msa_fp.name, db=self.dummy_db )

		msa_fp.close()

		return profile_fn
	"""

	def build_profile( self ) :
		'''
		This function builds a profile for the sequence using blastpgp.
		BlastMSA object for the blast hit result will be returned.
		'''

		print "Building profile for ", self.query_fn, "..."

		if self.dummy_db :
			self.build_temporary_dummy_db() 
		
		msa_fp, temp_name = tempfile.mkstemp()
		msa_fp = open( temp_name, 'w' )
		self.build_psiblast_alignment_input( msa_fp )
		msa_fp.flush()
			
		runner = PsiblastRunner( self.query_fn )
		profile_fn = runner.run_blast_for_profile( temp_name, db=self.dummy_db )

		msa_fp.close()
		os.system( 'rm -f '+ temp_name )

		return profile_fn



	def build_temporary_dummy_db( self ) :
		'''
		Build temporary dummy database for blastpgp to make the 

		build_temporary_dummy_db() returns the absolute pathname for the dummy_db
		'''
		
		self.temporary_dummy_db_dir = tempfile.mkdtemp()
		db_name = 'dummy_db'#self.query_fn.split('/')[-1]
		db_fn = self.temporary_dummy_db_dir + '/' + db_name
		shutil.copy( self.query_fn, db_fn )
		self.dummy_db = db_fn

		cwd = os.getcwd()

		os.chdir( self.temporary_dummy_db_dir )
		os.system( formatdb_program + ' -i ' + self.dummy_db )

		os.chdir( cwd )

		print 'temporary db built', self.dummy_db

		return self.dummy_db
		
	
	def remove_temporary_dummy_db( self ) :
		'''
		Removes temporary dummy db files and the temporary directory contains them.
	
		remove_temporary_dummy_db() returns None
		'''
		if self.dummy_db :
			os.system( 'rm -rf ' + self.temporary_dummy_db_dir )
		self.dummy_db = ''
		
		

	def psiblast_purge( self, neighbor_msa ) :
		#effective length of the database
		#effective_length = neighbor_msa.effective_length_of_the_database
		effective_length  = 5000
		
		hit_db = self.build_blast_database()
		
		neighbor_runner = PsiblastRunner( neighbor_msa.query_fn, output_fn_base=hit_db ) 
		neighbor_runner.db = hit_db
		neighbor_runner.additional_options = ' -z %s '% effective_length
		neighbor_runner.iteration = 1

		overlapping_msa = neighbor_runner.run_next_iteration( neighbor_msa )
		self.remove_overlapped_hit_regions( overlapping_msa )

		
	def count_hsps( self ) :
		i = 0 
		for hit in self.hits :
			for hsp in hit.hsps :
				i+= 1
		return i

	def write_hit_fasta( self, fp ) :
		for hit in self.hits :
			hit.write_hit_fasta(fp) 

	def get_hit_gi_set( self ) :
		id_set = set()
		for hit in self.hits :
			id_set.add( hit.gi )

		return id_set

	def build_gi2hit( self ) :
		'''
		This function make unique gi index called gi2hit.
		As a byproduct, this function also make compaction in the hit list
		to make all hits are uniquely found by a give gi number.
		'''
		for hit in self.hits[:] :
			if hit.gi in self.gi2hit :
				self.gi2hit[ hit.gi ].add_hit_as_hsp( hit ) 
				self.hits.remove( hit )
			else :
				self.gi2hit[ hit.gi ] = hit

	def get_hit( self, gi ) :
		if not self.gi2hit :
			self.build_gi2hit()
			
		if gi in self.gi2hit :
			return self.gi2hit[gi]
		else :
			return None

	def remove_overlapped_hit_regions( self, blastmsa ) :
		"""
		This function returns a list of domains and their range of overlaps to the given domain.
		"""
		
		msa_gi_set1 = self.get_hit_gi_set()
		msa_gi_set2 = blastmsa.get_hit_gi_set()

		if verbose :
			print   "msa_gi_set1", msa_gi_set1
			print   ""
			print  "msa_gi_set2", msa_gi_set2

		common_gis = msa_gi_set2.intersection( msa_gi_set1 )
		if verbose :
			print 'common_gis:', common_gis

		for gi in common_gis :
			if verbose:
				print gi, "will be purged"
			hit1 = self.get_hit( gi )
			hit2 = blastmsa.get_hit( gi )
		
			hit1.remove_overlapped_hit_region( hit2 )


	def build_blast_database( self ) :
		db_name = self.blast_result_fn + ".hit.db"
		fp = open( db_name, 'w' )
		
		self.write_hit_fasta( fp )
		fp.close()

		os.system( formatdb_program+' -i ' + db_name  )
	
		return db_name
	
		
	def parse_from_fp( self, fp, psiblast_flag=1 ) :
		self.blast_result_fp = fp
		self.parse( use_fp = 1, psiblast_flag=psiblast_flag )

	def parse_from_file( self, blast_result_fn ) :
		self.blast_result_fn = blast_result_fn
		self.parse()

	def get_gi_set( self ) :
		return set( [ hit.gi for hit in self.hits ] )

	def get_query( self, fn ) :
		fp = open( fn ) 
		query_seq = ''
		annotation = ''
		comment_line = fp.readline()
		if comment_line[0] == '>' :
			annotation = comment_line[1:].split()[0]
		else :
			annotation = 'Query'
			query_seq += comment_line.strip()
	
		for line in fp :
			if line[0] == '>' :
				break
			query_seq += line.strip()

		return annotation, query_seq

	def build_psiblast_alignment_input( self, write_fp ) :
		"""
		Build PSIBLAST input MSA.
		This function does not make any modification on the psiblast result.
		"""
		write_fp.write( '%-20s %s\n'%( self.annotation[:20], self.query_seq) )
		for hit in self.hits :
			aligned_seq = hit.get_aligned_hit_sequence( len(self.query_seq) )
			write_fp.write( aligned_seq )
		write_fp.flush()

	def build_psiblast_alignment_input_combined_hsps( self, write_fp ) :
		"""
		Build PSIBLAST input MSA.
		This function does not make any modification on the psiblast result.
		"""
		write_fp.write( '%-20s %s\n'%( self.annotation[:20], self.query_seq) )
		for hit in self.hits :
			aligned_seq = hit.get_aligned_hit_sequence_combined( len(self.query_seq) )
			write_fp.write( aligned_seq )
		write_fp.flush()

	def set_combined_hsps( self ) :
		'''
		Be careful!! this function just changes build psiblast alignment input to build psiblast alignment input combined hsps.
		'''
		self.build_psiblast_alignment_input = self.build_psiblast_alignment_input_combined_hsps


	def remove_overlapping_hsps( self ) :
		"""
		This function makes modification on the psiblast result by removing
		possibly contaminated Hits.
		Experiments #3
		"""
		for hit in self.hits[:] :
			hit.remove_overlapping_hsps()
			if not hit.hsps :
				self.hits.remove( hit )

	def purge_overlapping_hsps( self, positions, pssm ) :
		'''
		This function makes modification ont he psiblast result by removing
		possibly contaminated regions in the hsps.
		Check which portion of the HSP are good if the given hsps are overlapping.
		Experiments #4
		'''

		print '###Start to purge overlapping hits...'

		for hit in self.hits[:] :
			hit.purge_overlapping_hsps( positions, pssm ) 

			###################################
			#remove the hit if selection failed
			###################################
			if not hit.hsps:
				self.hits.remove( hit )


	def to_string( self ) :
		s = ""
		for hit in self.hits :
			s += hit.to_string()

		return s

	def parse( self, use_fp = 0, psiblast_flag = 1 ) :
		blast_hit = BlastHit()
		fp = None
		if use_fp :
			fp = self.blast_result_fp
		else :
			fp = open( self.blast_result_fn )

		if psiblast_flag == 1 :
			best_iteration_start, selected_iteration = select_last_iteration( fp )

			print >>sys.stderr, "Iteration", selected_iteration, " is selected!"
			print >>sys.stderr, "Starts at", best_iteration_start, "!"
		
			fp.seek( best_iteration_start ) #pointing to the start position of final iteration
					

		if verbose :
			print '##### after psi-blast result parsing routine ######'
			print 'current file pointer, fp.tell():', fp.tell()

		parse_flag = blast_hit.parse( fp )

		if verbose :
			print 'first blast hit parsing is done!'
			print 'parse_flag:',parse_flag
			print 'blast_hit.gi:', blast_hit.gi


		while parse_flag == 1 :
			self.hits.append( blast_hit )
			blast_hit = BlastHit()
			parse_flag = blast_hit.parse( fp )

			if parse_flag == -2 :
				if verbose :
					print 'End of Blast Result is processed!'
				self.hits.append( blast_hit )
				break

			if verbose :
				print 'parse_flag @ BlastMSA parse :', parse_flag
			
			

		self.parsing_status = parse_flag

		if parse_flag == -1 :

			print >>sys.stderr, "WARNING: Erroneous Parsing of a Blast Hit.", parse_flag
			print >>sys.stderr, "This possibly indicate premature finish of a Blast/PSI Blast result."
			print >>sys.stderr, fp.readline()

			return False

		elif parse_flag == 0 or parse_flag == -2 :
			pass #everything is done

		elif parse_flag == 1 :
			#something is not complete
			print >>sys.stderr, "WARNING: Hit records are all correctly parsed, but the result may not contain all data."

		else :
			print >>sys.stderr, "Error! This message should not be seen!"

		#Need to parse effective database size
		#code will be here!!
			
		fp.close()

		return True

class BlastMSAm6( BlastMSA ) :
	def parse( self, use_fp=0, psiblast_flag=1 ) :
		fp = None
		if use_fp :
			fp = self.blast_result_fp
		else :
			fp = open( self.blast_result_fn )

		if psiblast_flag == 1 :
			best_iteration_start, selected_iteration = select_last_iteration( fp )

			print >>sys.stderr, "Iteration", selected_iteration, " is selected!"
			print >>sys.stderr, "Starts at", best_iteration_start, "!"
		
			fp.seek( best_iteration_start ) #pointing to the start position of final iteration

		#####################
		#BlastMSAm6 only variables
		self.hit_ids = []
		self.hsps = []
		self.hit_starts = []
		self.hit_ends = []
		#####################

		
		if verbose :
			print "@BlastMSAm6.parse: start to read iteration lines"

		line = fp.readline() 
		while line and line[:len(psiblast_start_m6)] != psiblast_start_m6 :
			line = fp.readline()
			
	
		if verbose :
			print "@BlastMSAm6.parse: found correct start point of m6 alignments"

		first_block = 1 #flag for the first block
		hsp_num = 0
		end_code = 1 #abnormal end

		while line :
			if line[:len(psiblast_end_of_result_marker)] == psiblast_end_of_result_marker :
				end_code = -2 #normal end
				break
			elif line[:len(psiblast_end_of_result_iteration)] == psiblast_end_of_result_iteration :
				end_code = -2
				break 

			l = line.split()
			
			if len(l) == 2 :
				id = l[0] 
				aln = l[1]
				start = 9999999
				end = -1

			elif len(l)== 3 :
				id = l[0] 
				aln = l[2]
				start = int( l[1] )
				end = int( l[1] )
				
			elif len(l) == 4 :
				id = l[0] 
				aln = l[2]
				start = int(l[1])
				end = int(l[3])

			elif len(l) == 0 :
				first_block = 0
				line = fp.readline()
				hsp_num = 0
				continue
			else :
				print >>sys.stderr, "Error! This should be not seen!"
				
			if first_block :
				self.hit_ids.append(id)
				self.hsps.append(aln)
				self.hit_starts.append( start )
				self.hit_ends.append( end )
			else :
				if self.hit_ids[hsp_num] == id :
					self.hsps[hsp_num] = self.hsps[hsp_num] + aln
					#getting hit boundary info
					if self.hit_starts[hsp_num] > start :
						self.hit_starts[hsp_num] = start
					if self.hit_ends[hsp_num] < end :
						self.hit_ends[hsp_num] = end
				else :
					print >>sys.stderr, "Error while parsing m6 psiblast/blast result:", line
			
			line = fp.readline()
			hsp_num += 1

		if end_code == -2 :
			print "Parsing is done!"
		else :
			print "Parsing is not done correctly!", end_code


		#additional checking for query coverage!
		if self.hsps :
			query_start = self.hit_starts[0]
			query_end = self.hit_ends[0]

			#adjusting for the blast output problem!!
			#may start with 0 and bigger than the actual sequence length
			start = max (query_start, 1 ) 
			end = min( query_end, len(self.query_seq) )
	
			if len(self.query_seq) == end-start+1 :
				print "Query is fully coverered!"
			else :
				print "WARNING: Query is not fully covered!", end-start+1
			

	def build_psiblast_alignment_input( self, write_fp ) :
		#check intergrity of alingment result
		#and only OK sequences will be written!
		if self.hsps :
			aln_length = len(self.hsps[0])
			self.hit_ids[0] = self.annotation[:20]

			for name, aln in zip( self.hit_ids, self.hsps ) :
				if len(aln) != aln_length :
					print >>sys.stderr, "Erroneous results in parsing!", aln_length, len(aln),'\n'+ aln
					continue

				print >>write_fp, '%-20s %s'% (name, aln)
		else :
			print "No sequence are found!"
			write_fp.write( '%-20s %s\n'%( self.annotation[:20], self.query_seq) )

	def get_list_of_non_gapped_positions( self, gapped_sequence ) :
		nongapped_positions = []
		within_nongap = 0
		
		for i, c in enumerate( gapped_sequence ) :
			if within_nongap == 0 :
				if c == '-' :
					continue
				else :
					nongap_start = i
					within_nongap = 1
					continue
			else :
				if c == '-' :
					within_nongap = 0
					nongapped_positions.append( (nongap_start, i) )
					continue
				else :
					continue
		else :
			if within_nongap == 0 :
				pass
			else :
				nongapped_positions.append( (nongap_start, i+1 ) )
			
		return nongapped_positions

	def get_non_gapped_sequence( self, gapped_seq, nongapped_positions ) :
		str_fp = cStringIO.StringIO()
		[ str_fp.write( gapped_seq[i:j] ) for i,j in nongapped_positions ]
	
		return str_fp.getvalue()

	def build_psiblast_alignment_input_without_query_gaps( self, write_fp ) :
		if self.hsps :
			aln_length = len(self.hsps[0])
			self.hit_ids[0] = self.annotation[:20]
			nongapped_positions = self.get_list_of_non_gapped_positions( self.hsps[0] )

			for name, aln in zip( self.hit_ids, self.hsps ) :
				if len(aln) != aln_length :
					print >>sys.stderr, "Erroneous results in parsign!", aln_length, len(aln),'\n'+ aln
					continue

				print >>write_fp, '%-20s %s'% ( name, self.get_non_gapped_sequence(aln, nongapped_positions) )

		else :
			print "No sequence are found!"
			write_fp.write( '%-20s %s\n'%( self.annotation[:20], self.query_seq) )



class ScoreMat :
	def __init__( self, query_len=0 ) :
		self.length = query_len

		self.columns = [] #PSSM
		self.blosum_mat = {} #Blosum

		self.residue_map = {}

		self.gop = 11 #gap opening panelty
		self.gep = 1  #gap extending panelty


	def build_pssm( self, msa ) :
		'''
		Generate PSSM for a given blast msa.
		1. Run psiblast for a dummy db.
		2. read in the pssm file.
		'''
		fn = msa.build_profile()
		self.read_pssm_fn( fn )


	def combine( self, score_mat ) :
		'''
		Add given score_mat as a following part of PSSM
		'''
		pass
		

	def read_pssm_fn( self, fn ) :

		print >>sys.stderr, '#Start to read profile', fn

		fp = open( fn )

		fp.readline() #read one line
		line = fp.readline()
		header = fp.readline().split()[:20]

		for i, a in enumerate(header) :
			self.residue_map[ a.upper() ] = i
			self.residue_map[ a.lower() ] = i

		for i, line in enumerate(fp) :
			if line == '\n' :
				break
			if verbose :
				print line

			line = line.split()

			index, residue = line[:2]
			if verbose :
				print 'index:', index, 'residue:', residue

			if i != int(index)-1 :
				print >>sys.stderr, "WARNING: ", fn, "has problem!", i, index
		
			self.columns.append( [ int(s) for s in line[2:22] ] )
			#vector = []
			#for s in line [2:22] :
				#vector.append( int(s) )
			#self.columns.append( vector )

		if self.length != len(self.columns) :
			print >>sys.stderr, 'WARNING! the length of query and profile does not match!'


	def pssm( self, residue_number, query_residue_type, residue_type ) :
		#query residue type is not used!!
		if residue_type in self.residue_map :
			s = self.columns[residue_number][ self.residue_map[residue_type] ]
			if verbose :
				print '%s pssm[%s][%s]=%s' % (residue_number,query_residue_type, residue_type,s)
			return s
		else :
			print >>sys.stderr, 'Warning! residue_number:', residue_number, 'query_residue_type:', query_residue_type, 'residue_type:', residue_type
			return 0.0
		


	def cal_pssm_score( self, hsp, start_pos=0, end_pos=0, matrix='pssm' ) :
		'''
		This function calculate pssm score for given hsp.
		Start and end are not query based residue numbering.
		'''

		score_mat = self.pssm
		if matrix == 'blosum' :
			score_mat = self.blosum
		elif self.length == 0 :
			score_mat = self.blosum

		#By default tihs function calculate score for whole area.
		if start_pos == 0 and end_pos == 0 :
			start_pos = hsp.query_start_index
			end_pos = hsp.query_end_index

		start = hsp.convert_position_to_real_index( hsp.query, hsp.query_start_index, start_pos  )
		end = hsp.convert_position_to_real_index( hsp.query, hsp.query_start_index, end_pos  )

		if verbose :
			print "Calculating scores using pssm...", start, '/', len(hsp.query), end, '/', len(hsp.hit)
			print 'query:', hsp.query[start:end]
			print 'hit  :', hsp.hit[start:end]

		gap_flag = 0

		#if the start position is gap opening, mark the gap_start
		if hsp.hit[start] == '-' or hsp.query[start] == '-' :
			if ( start != 0 and hsp.hit[start-1] != '-' ) or ( start != 0 and hsp.query[start-1] != '-' ) :
				gap_flag = 0
			else :
				gap_flag = 1

		
		score = 0
		query_resnum = hsp.query_start_index - 1
		for q, h in zip(hsp.query[start:end],hsp.hit[start:end]) :
			if q.isalpha() :
				query_resnum += 1
			if q == '-' or h == '-' :
				if gap_flag :
					score -= self.gep
				else :
					score -= self.gop
					gap_flag = 1
			else :
				score += score_mat( query_resnum, q, h ) 
				gap_flag = 0

			if verbose :
				print 'score:', score
			
		
		return score
			

		
	def combine_optimal_by_pssm( self, hsp1, hsp2 ) :
		'''
		This function finds optimal intersection point between two intersecting hsps.
		'''
		new_hsp = copy.deepcopy( hsp1 )
		
		pass

	def blosum( self, resnum, res1, res2 ) :
		#resum is not used for blosum
		#place holoder for pssm score. :)
		score = 0.0
		try :
			score = self.blosum_mat[res1.upper(), res2.upper()]
		except :
			print >>sys.stderr, "WARNING:", res1.upper(), "or", res2.upper(), "not found in BLOSUM Mat."
		return score

	def set_blosum_mat( self, fn='' ) :
		mat_str = ''
		if fn :
			mat_str = open(fn).read()
			
		else : #default 
			mat_str = '''#  Matrix made by matblas from blosum62.iij
#  * column uses minimum score
#  BLOSUM Clustered Scoring Matrix in 1/2 Bit Units
#  Blocks Database = /data/blocks_5.0/blocks.dat
#  Cluster Percentage: >= 62
#  Entropy =   0.6979, Expected =  -0.5209
   A  R  N  D  C  Q  E  G  H  I  L  K  M  F  P  S  T  W  Y  V  B  Z  X  *
A  4 -1 -2 -2  0 -1 -1  0 -2 -1 -1 -1 -1 -2 -1  1  0 -3 -2  0 -2 -1  0 -4 
R -1  5  0 -2 -3  1  0 -2  0 -3 -2  2 -1 -3 -2 -1 -1 -3 -2 -3 -1  0 -1 -4 
N -2  0  6  1 -3  0  0  0  1 -3 -3  0 -2 -3 -2  1  0 -4 -2 -3  3  0 -1 -4 
D -2 -2  1  6 -3  0  2 -1 -1 -3 -4 -1 -3 -3 -1  0 -1 -4 -3 -3  4  1 -1 -4 
C  0 -3 -3 -3  9 -3 -4 -3 -3 -1 -1 -3 -1 -2 -3 -1 -1 -2 -2 -1 -3 -3 -2 -4 
Q -1  1  0  0 -3  5  2 -2  0 -3 -2  1  0 -3 -1  0 -1 -2 -1 -2  0  3 -1 -4 
E -1  0  0  2 -4  2  5 -2  0 -3 -3  1 -2 -3 -1  0 -1 -3 -2 -2  1  4 -1 -4 
G  0 -2  0 -1 -3 -2 -2  6 -2 -4 -4 -2 -3 -3 -2  0 -2 -2 -3 -3 -1 -2 -1 -4 
H -2  0  1 -1 -3  0  0 -2  8 -3 -3 -1 -2 -1 -2 -1 -2 -2  2 -3  0  0 -1 -4 
I -1 -3 -3 -3 -1 -3 -3 -4 -3  4  2 -3  1  0 -3 -2 -1 -3 -1  3 -3 -3 -1 -4 
L -1 -2 -3 -4 -1 -2 -3 -4 -3  2  4 -2  2  0 -3 -2 -1 -2 -1  1 -4 -3 -1 -4 
K -1  2  0 -1 -3  1  1 -2 -1 -3 -2  5 -1 -3 -1  0 -1 -3 -2 -2  0  1 -1 -4 
M -1 -1 -2 -3 -1  0 -2 -3 -2  1  2 -1  5  0 -2 -1 -1 -1 -1  1 -3 -1 -1 -4 
F -2 -3 -3 -3 -2 -3 -3 -3 -1  0  0 -3  0  6 -4 -2 -2  1  3 -1 -3 -3 -1 -4 
P -1 -2 -2 -1 -3 -1 -1 -2 -2 -3 -3 -1 -2 -4  7 -1 -1 -4 -3 -2 -2 -1 -2 -4 
S  1 -1  1  0 -1  0  0  0 -1 -2 -2  0 -1 -2 -1  4  1 -3 -2 -2  0  0  0 -4 
T  0 -1  0 -1 -1 -1 -1 -2 -2 -1 -1 -1 -1 -2 -1  1  5 -2 -2  0 -1 -1  0 -4 
W -3 -3 -4 -4 -2 -2 -3 -2 -2 -3 -2 -3 -1  1 -4 -3 -2 11  2 -3 -4 -3 -2 -4 
Y -2 -2 -2 -3 -2 -1 -2 -3  2 -1 -1 -2 -1  3 -3 -2 -2  2  7 -1 -3 -2 -1 -4 
V  0 -3 -3 -3 -1 -2 -2 -3 -3  3  1 -2  1 -1 -2 -2  0 -3 -1  4 -3 -2 -1 -4 
B -2 -1  3  4 -3  0  1 -1  0 -3 -4  0 -3 -3 -2  0 -1 -4 -3 -3  4  1 -1 -4 
Z -1  0  0  1 -3  3  4 -2  0 -3 -3  1 -1 -3 -1  0 -1 -3 -2 -2  1  4 -1 -4 
X  0 -1 -1 -1 -2 -1 -1 -1 -1 -1 -1 -1 -1 -1 -2  0  0 -2 -1 -1 -1 -1 -1 -4 
* -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4 -4  1 
'''
		lines = mat_str.split('\n')
		
		aa_list = lines[6].split()

		for i, l in enumerate(lines[7:7+len(aa_list)]) :
			l = l.split()
			a = l[0]
			scores = l[1:]
			for j, score in enumerate(scores) :
				self.blosum_mat[ a, aa_list[j] ] = float(score)
			
			

class PsiblastRunner :
	def __init__( self, fn, output_fn_base='' ) :
		self.iteration = 8
		self.cpu = number_of_cpus
		self.inclusionevalue = 0.001
		self.final_evalue = 0.001
		self.input_fn = fn
		self.db = default_db
		self.number_of_alignments_to_show = 50000 #b option
		self.summary_length = 0 #v option

		self.blast_program = blast_program
		self.current_iteration = 0

		if output_fn_base :
			self.output_fn_base = output_fn_base
		else :
			self.output_fn_base = remove_fasta_extension(fn.split('/')[-1].strip()) #clean up the output name!!

		self.output_fn_template = self.output_fn_base + ".%d.out"
		self.input_alignment_fn_template = self.output_fn_base + '.%d.aln'
		self.additional_options = ''

	def set_composition_bias_statistics( self, n ) :
		self.additional_options += ' -t %s'%n

	def set_local_realign( self ) :
		self.additional_options += ' -s T'

	def run_psiblast_original( self ) :
		"""
		This is the psiblast running function.
		This function is different from run_psiblast function by using original Psiblast with -j option.
		"""
		parameters = ' -I T -h %f -e %f %s '% (self.inclusionevalue, self.inclusionevalue, self.additional_options)
		if self.db :
			parameters += '-d %s ' % self.db
		if self.cpu > 1 :
			parameters += '-a %s ' % self.cpu

		parameters += ' -v %d -b %d ' % ( self.summary_length, self.number_of_alignments_to_show )
		
		output_fn = self.output_fn_template % (self.iteration)
		parameters += ' -o %s -j %d' % (output_fn, self.iteration)

		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + parameters
		print >>sys.stderr, cmd
		os.system( cmd )
		self.current_iteration = self.iteration - 1
		return output_fn


	def run_psiblast( self ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration.
		"""
		initial_blast_result = self.run_initial_blast()
		pre_msa = BlastMSA( self.input_fn, initial_blast_result )
		cur_msa = self.run_next_iteration( pre_msa )

		pre_gi_set = len( pre_msa.get_gi_set() )
		cur_gi_set = len( cur_msa.get_gi_set() )
		
		if pre_gi_set == cur_gi_set :
			print >>sys.stderr, "Converged!"
			return cur_msa

		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration( pre_msa )

			pre_gi_set = len( pre_msa.get_gi_set() )
			cur_gi_set = len( cur_msa.get_gi_set() )
			if pre_gi_set == cur_gi_set :
				print >>sys.stderr, "Converged!"
				break

			elif pre_gi_set > cur_gi_set :
				print >>sys.stderr, "Converged!"
				return pre_msa

		#building_alignment output for the last run
                input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)
                fp = open( input_alignment_fn, 'w' )
                cur_msa.build_psiblast_alignment_input( fp )
                fp.close()

		return cur_msa

	def run_psiblast_checkpoint( self ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration with checkpoint.
		"""
		initial_blast_result = self.run_initial_blast_checkpoint()
		pre_msa = BlastMSA( self.input_fn, initial_blast_result )
		cur_msa = self.run_next_iteration_checkpoint( pre_msa )

		pre_gi_set = len( pre_msa.get_gi_set() )
		cur_gi_set = len( cur_msa.get_gi_set() )
		
		if pre_gi_set == cur_gi_set :
			print >>sys.stderr, "Converged!"
			return cur_msa

		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration_checkpoint( pre_msa )

			pre_gi_set = len( pre_msa.get_gi_set() )
			cur_gi_set = len( cur_msa.get_gi_set() )
			if pre_gi_set == cur_gi_set :
				print >>sys.stderr, "Converged!"
				break

			elif pre_gi_set > cur_gi_set :
				print >>sys.stderr, "Converged!"
				return pre_msa

		#building_alignment output for the last run
                input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)
                fp = open( input_alignment_fn, 'w' )
                cur_msa.build_psiblast_alignment_input( fp )
                fp.close()

		return cur_msa



	def run_psiblast_m6( self ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration.
		"""
		initial_blast_result = self.run_initial_blast_m6()
		pre_msa = BlastMSAm6( self.input_fn, initial_blast_result )
		cur_msa = self.run_next_iteration_m6( pre_msa )

		pre_gi_set = len( pre_msa.get_gi_set() )
		cur_gi_set = len( cur_msa.get_gi_set() )
		
		if pre_gi_set == cur_gi_set :
			print >>sys.stderr, "Converged!"
			return cur_msa

		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration_m6( pre_msa )

			pre_gi_set = len( pre_msa.get_gi_set() )
			cur_gi_set = len( cur_msa.get_gi_set() )
			if pre_gi_set == cur_gi_set :
				print >>sys.stderr, "Converged!"
				return cur_msa
			elif pre_gi_set > cur_gi_set :
				print >>sys.stderr, "Converged!"
				return pre_msa

		return cur_msa

	def run_psiblast_mafft( self ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration.
		"""
		initial_blast_result = self.run_initial_blast()
		pre_msa = BlastMSA( self.input_fn, initial_blast_result )
		cur_msa = self.run_next_iteration_mafft( pre_msa )
		

		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration_mafft( pre_msa )

		return cur_msa



	def run_psiblast_back_psiblasting_removing( self, neighbor_msas=[]  ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration.
		Also this version contains msas that would be used in back psiblast.
		"""

		initial_blast_result = self.run_initial_blast()
		pre_msa = BlastMSA( self.input_fn, initial_blast_result )
		for neighbor_msa in neighbor_msas :
			pre_msa.psiblast_purge( neighbor_msa )

		cur_msa = self.run_next_iteration( pre_msa )
		for neighbor_msa in neighbor_msas :
			cur_msa.psiblast_purge( neighbor_msa )


		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration( pre_msa )
			for neighbor_msa in neighbor_msas :
				cur_msa.psiblast_purge( neighbor_msa )

			pre_gi_set = len( pre_msa.get_gi_set() )
			cur_gi_set = len( cur_msa.get_gi_set() )
			if pre_gi_set == cur_gi_set :
				print >>sys.stderr, "Converged!"
				break

			elif pre_gi_set > cur_gi_set :
				print >>sys.stderr, "Converged!"
				return pre_msa

		#building_alignment output for the last run
                input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)
                fp = open( input_alignment_fn, 'w' )
                cur_msa.build_psiblast_alignment_input( fp )
                fp.close()

		return cur_msa

	def run_psiblast_without_overlapping_hsps( self ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration.

		This function is developed for blast purging idea #3.
		"""
		initial_blast_result = self.run_initial_blast()
		pre_msa = BlastMSA( self.input_fn, initial_blast_result )
		pre_msa.remove_overlapping_hsps() #purge blast alignment
		pre_msa.set_combined_hsps() #put hsps together

		cur_msa = self.run_next_iteration( pre_msa )
		cur_msa.remove_overlapping_hsps() #purge blast alignment
		cur_msa.set_combined_hsps() #put hsps together
		
		
		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration( pre_msa )
			cur_msa.remove_overlapping_hsps() #purge blast alignment
			cur_msa.set_combined_hsps() #put hsps together

			pre_gi_set = len( pre_msa.get_gi_set() )
			cur_gi_set = len( cur_msa.get_gi_set() )

			if pre_gi_set == cur_gi_set :
				print >>sys.stderr, "Converged!"
				break 

			elif pre_gi_set > cur_gi_set :
				print >>sys.stderr, "Converged!"
				return pre_msa

		#building_alignment output for the last run
                input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)
                fp = open( input_alignment_fn, 'w' )
                cur_msa.build_psiblast_alignment_input( fp )
                fp.close()

		return cur_msa


	def run_psiblast_purged_overlapping_hsps( self, inserted_pos=[], pssm1=None ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration.

		This function is developed for blast purging idea #4.
		purge_overlapping_hsps should be implemented
		"""
		initial_blast_result = self.run_initial_blast()
		pre_msa = BlastMSA( self.input_fn, initial_blast_result )

		#build pssm
		pssm = pssm1
		if not pssm :
			pssm = ScoreMat() #pre_msa.build_pssm()
			pssm.set_blosum_mat()

		pre_msa.purge_overlapping_hsps( inserted_pos, pssm ) #purge blast alignment
		pre_msa.set_combined_hsps() #put hsps together


		cur_msa = self.run_next_iteration( pre_msa )

		pssm = pssm1
		if not pssm :
			pssm = pre_msa.build_pssm()

		cur_msa.purge_overlapping_hsps( inserted_pos, pssm ) #purge blast alignment
		cur_msa.set_combined_hsps() #put hsps together

		pre_gi_set = len( pre_msa.get_gi_set() )
		cur_gi_set = len( cur_msa.get_gi_set() )


		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration( pre_msa )

			pssm = pssm1
			if not pssm :
				pssm = pre_msa.build_pssm()

			cur_msa.purge_overlapping_hsps( inserted_pos, pssm ) #purge blast alignment
			cur_msa.set_combined_hsps() #put hsps together

			pre_gi_set = len( pre_msa.get_gi_set() )
			cur_gi_set = len( cur_msa.get_gi_set() )

			if pre_gi_set == cur_gi_set :
				print >>sys.stderr, "Converged!"
				break

			elif pre_gi_set > cur_gi_set :
				print >>sys.stderr, "Converged!"
				return pre_msa

		#building_alignment output for the last run
                input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)
                fp = open( input_alignment_fn, 'w' )
                cur_msa.build_psiblast_alignment_input( fp )
                fp.close()

		return cur_msa

	def run_psiblast_purged_overlapping_hsps_with_backblasting( self, inserted_pos=[], neighbor_msas=[], pssm1=None ) :
		"""
		This is psi-blast like iterative blast running function.
		This is not original psiblast but run blast n iteration.

		This function is developed for blast purging idea #4.
		purge_overlapping_hsps should be implemented
		"""
		initial_blast_result = self.run_initial_blast()
		pre_msa = BlastMSA( self.input_fn, initial_blast_result )

		#build pssm
		pssm = pssm1
		if not pssm :
			pssm = ScoreMat() #pre_msa.build_pssm()
			pssm.set_blosum_mat()

		pre_msa.purge_overlapping_hsps( inserted_pos, pssm ) #purge blast alignment
		
		#second purging with backblasting!!
		for neighbor_msa in neighbor_msas :
			pre_msa.psiblast_purge( neighbor_msa )

		pre_msa.set_combined_hsps() #put hsps together


		cur_msa = self.run_next_iteration( pre_msa )

		pssm = pssm1
		if not pssm :
			pssm = pre_msa.build_pssm()

		cur_msa.purge_overlapping_hsps( inserted_pos, pssm ) #purge blast alignment

		#second purging with backblasting!!
		for neighbor_msa in neighbor_msas :
			cur_msa.psiblast_purge( neighbor_msa )

		cur_msa.set_combined_hsps() #put hsps together


		for i in xrange( 2, self.iteration ) :
			pre_msa = cur_msa
			cur_msa = self.run_next_iteration( pre_msa )

			pssm = pssm1
			if not pssm :
				pssm = pre_msa.build_pssm()

			cur_msa.purge_overlapping_hsps( inserted_pos, pssm ) #purge blast alignment
			#second purging with backblasting!!
			for neighbor_msa in neighbor_msas :
				cur_msa.psiblast_purge( neighbor_msa )

			cur_msa.set_combined_hsps() #put hsps together

			pre_gi_set = len( pre_msa.get_gi_set() )
			cur_gi_set = len( cur_msa.get_gi_set() )

			if pre_gi_set == cur_gi_set :
				print >>sys.stderr, "Converged!"
				break

			elif pre_gi_set > cur_gi_set :
				print >>sys.stderr, "Converged!"
				return pre_msa

		#building_alignment output for the last run
                input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)
                fp = open( input_alignment_fn, 'w' )
                cur_msa.build_psiblast_alignment_input( fp )
                fp.close()

		return cur_msa




	def run_initial_blast( self, options='' ) :

		parameters = ' %s -I T -h %f -e %f %s '% (options, self.inclusionevalue, self.inclusionevalue, self.additional_options)
		if self.db :
			parameters += '-d %s ' % self.db
		if self.cpu > 1 :
			parameters += '-a %s ' % self.cpu

		parameters += ' -v %d -b %d ' % ( self.summary_length, self.number_of_alignments_to_show )
		
		output_fn = self.output_fn_template % (self.current_iteration + 1)
		parameters += ' -o %s' % output_fn

		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + parameters
		print cmd
		os.system( cmd )
		self.current_iteration += 1
		return output_fn


	def run_initial_blast_checkpoint( self, options='' ) :

		parameters = ' %s -I T -h %f -e %f %s '% (options, self.inclusionevalue, self.inclusionevalue, self.additional_options)
		if self.db :
			parameters += '-d %s ' % self.db
		if self.cpu > 1 :
			parameters += '-a %s ' % self.cpu

		parameters += ' -v %d -b %d ' % ( self.summary_length, self.number_of_alignments_to_show )
		
		output_fn = self.output_fn_template % (self.current_iteration + 1)
		parameters += ' -o %s' % output_fn

		##########
		#first generate checkpoint file
		self.checkpoint_fn = output_fn + '.chk'
		checkpoint_parameters = parameters + ' -C %s -j 2' % self.checkpoint_fn

		cmd = self.blast_program + ' ' + '-i %s '% self.input_fn + checkpoint_parameters
		print cmd
		os.system( cmd )
		##########

		##########
		#then build normal blast output
		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + parameters
		print cmd

		os.system( cmd )
		##########

		self.current_iteration += 1
		return output_fn


	def run_initial_blast_m6( self, options='' ) :

		options=options + ' -m6'
		parameters = ' %s -I T -h %f -e %f %s '% (options, self.inclusionevalue, self.inclusionevalue, self.additional_options)
		if self.db :
			parameters += '-d %s ' % self.db
		if self.cpu > 1 :
			parameters += '-a %s ' % self.cpu

		parameters += ' -v %d -b %d ' % ( self.summary_length, self.number_of_alignments_to_show )
		
		output_fn = self.output_fn_template % (self.current_iteration + 1)
		parameters += ' -o %s' % output_fn

		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + parameters
		print cmd
		os.system( cmd )
		self.current_iteration += 1
		return output_fn

	def run_blast( self, input_msa_fn, db='', options='' ) :
		if db == '' :
			db = self.db

		parameters = ' %s -I T -h %f -e %f %s '% (options, self.inclusionevalue, self.inclusionevalue, self.additional_options)
		if self.db :
			parameters += '-d %s ' % db
		if self.cpu > 1 :
			parameters += '-a %s ' % self.cpu

		parameters += ' -v %d -b %d ' % ( self.summary_length, self.number_of_alignments_to_show )
		
		output_fn = self.output_fn_template % (self.current_iteration+1)
		parameters += ' -o %s' % output_fn
		parameters += ' -B %s' % input_msa_fn

		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + parameters
		print cmd
		os.system( cmd )
		self.current_iteration += 1
		return output_fn

	def run_blast_for_checkpoint( self, input_alignment_fn, db='', options='' ) :
		if db == '' :
			db = self.db

		parameters = ' %s -I T -h %f -e %f %s '% (options, self.inclusionevalue, self.inclusionevalue, self.additional_options)
		if self.db :
			parameters += '-d %s ' % db
		if self.cpu > 1 :
			parameters += '-a %s ' % self.cpu

		parameters += ' -v %d -b %d ' % ( self.summary_length, self.number_of_alignments_to_show )
		
		output_fn = self.output_fn_template % (self.current_iteration)
		parameters += ' -o %s' % output_fn
		parameters += ' -R %s' % self.checkpoint_fn #checkpoint file generate in previous step

		self.checkpoint_fn = output_fn + '.chk' #new checkpoint file!!
		checkpoint_parameters = parameters + ' -j 2 -C '+self.checkpoint_fn

		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + checkpoint_parameters
		print cmd
		os.system( cmd )

		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + parameters
		print cmd
		os.system( cmd )
		self.current_iteration += 1
		return output_fn


	def run_blast_for_profile( self, input_msa_fn, db = '', profile_fn='' ) :
		if db == '' :
			db = self.db
	
		if profile_fn == '' :
			profile_fn = input_msa_fn.split('/')[-1] + '.profile' 

		parameters = ' -I T -h %f -e %f %s '% (self.inclusionevalue, self.inclusionevalue, self.additional_options)
		if db :
			parameters += '-d %s ' % db

		if self.cpu > 1 :
			parameters += '-a %s ' % self.cpu

		parameters += ' -v %d -b %d ' % ( self.summary_length, self.number_of_alignments_to_show )
		parameters += ' -Q %s ' % ( profile_fn )
		
		####################
		#do not make output fn
		#output_fn = self.output_fn_template % (self.current_iteration+1)
		#parameters += ' -o %s' % output_fn
		####################
		parameters += ' -B %s' % input_msa_fn

		cmd = self.blast_program + ' ' + '-i %s '%self.input_fn + parameters + '> /dev/null'
		print cmd
		os.system( cmd )
		#self.current_iteration += 1
		return profile_fn


	def run_next_iteration( self, prev_msa ) :
		input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)

		fp = open( input_alignment_fn, 'w' )
		prev_msa.build_psiblast_alignment_input( fp )
		fp.close()

		new_output_fn = self.run_blast( input_alignment_fn )
		new_msa = BlastMSA( self.input_fn, new_output_fn )
	
		return new_msa

	def run_next_iteration_checkpoint( self, prev_msa ) :
		input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)

		fp = open( input_alignment_fn, 'w' )
		prev_msa.build_psiblast_alignment_input( fp )
		fp.close()

		new_output_fn = self.run_blast_for_checkpoint( input_alignment_fn )
		new_msa = BlastMSA( self.input_fn, new_output_fn )
	
		return new_msa

	def run_next_iteration_m6( self, prev_msa ) :
		input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)

		fp = open( input_alignment_fn, 'w' )
		prev_msa.build_psiblast_alignment_input( fp )
		fp.close()

		new_output_fn = self.run_blast( input_alignment_fn, options='-m6' )
		new_msa = BlastMSAm6( self.input_fn, new_output_fn )
	
		return new_msa


	def run_next_iteration_mafft( self, prev_msa ) :
		input_alignment_fn = self.input_alignment_fn_template % (self.current_iteration)

		fasta_fn = input_alignment_fn + '.fasta'
		fp = open( fasta_fn, 'w' )
		fp.write( open(self.input_fn).read() )
		prev_msa.write_hit_fasta( fp )
		fp.close()

		if prev_msa.count_hsps() > 20000 :
			os.system( 'mafft --parttree --clustalout ' + fasta_fn + """|awk 'NR>3{print}' > """ + input_alignment_fn )
		else :
			os.system( 'mafft --auto --clustalout ' + fasta_fn + """|awk 'NR>3{print}' > """ + input_alignment_fn )

		new_output_fn = self.run_blast( input_alignment_fn )
		new_msa = BlastMSA( self.input_fn, new_output_fn )
	
		return new_msa


############################
# utility functions
############################

def read_fasta_file( fn ) :
	fp = open( fn )
	annotation = fp.readline()[1:-1]
	seq = ''.join( [ l.strip() for l in fp ] )

	gi = ""
	if annotation[:4] == '>gi|' :
		gi = annotation.split('|')[1]
	fp.close()

	return gi, annotation, seq

def remove_fasta_extension( fn ) :
	names = fn.split('.')
	extensions = ['fa', 'fasta', 'fas', 'pdb', 'ent']

	#no extension in filename!!
	if len(names) < 2 :
		return fn

	new_name = '.'.join( names[:-1] )
	if names[-1] in extensions :
		pass
	else :
		new_name = new_name + '.' + names[-1]

	return new_name

def convert_user_range2python_ranges( range_string ) :
	'''
	convert input fasta sequence ranges of domains into 
	range lists can be used in fasta slices.
	1-10,20-30
	'''
	if range_string == 'all' :
		return range_string

	ranges = range_string.split(',')
	new_ranges = []
	for range in ranges :
		try :
			s, e = range.split('-')
			s = int(s)
			e = int(e)
			
		except :
			print >>sys.stderr, "Error! Domain range is not defined correctly!", range_string
			sys.exit()
	
		#s convert s-1, 0-based string
		# e should be remain still, for slicing index
		new_ranges.append( [s-1, e] )

	return new_ranges

def generate_inserted_ranges( user_range ) :
	'''
	from user_range such as "1-10,20-30",
	find insertions like "11-19"
	'''
	
	inserted_user_ranges = []
	ranges = convert_user_range2python_ranges( user_range )
	for  r1, r2 in zip( ranges[:-1], ranges[1:] ) :
		s = str( r1[1]+1 ) + '-' + str( r2[0] )
		inserted_user_ranges.append( s )

	return inserted_user_ranges
	

def get_subsequence_from_ranges( seq, range_string ) :
	'''
	ranges should be [ [0,10], [10,20] ... ]
	'''
	ranges = convert_user_range2python_ranges( range_string )

	new_seq_list = [ seq[i:j] for i,j in ranges ]

	cur_len = 0
	range_strings = []
	for s in new_seq_list :
		range_strings.append( str(cur_len) + '-' + str(cur_len + len(s)) )
		cur_len += len(s)
	
	new_seq = ''.join ( new_seq_list )

	return new_seq, range_strings


def write_fasta_file( fn, seq, name="Query", overwrite=0) :
	if not overwrite and os.path.exists( fn ) :
		print >>sys.stderr, "Error!", fn, "already exists"
		sys.exit()
		
	fp = open( fn, 'w' )

	str = ">%s\n%s\n" % (name, seq)
	fp.write( str )
	fp.close()
	

def fill_gaps( str, length ) :
	if len(str) == length :
		return str
	elif len(str) == 0 :
		return '-'*length
	else :
		print >>sys.stderr, "WARNING! The string should be All or None", str, length
		return '-'*length


def combine_psiblast_input_files_horizontally( fn_list, title='Query' ) :
	'''
	Combine strings in the files in fn_list horizontally
	then return StringIO object.

	Needed for join fragment idea!!
	'''

	to_fp = cStringIO.StringIO()

	if not fn_list :
		return None
	

	fps = [ open(fn) for fn in fn_list]
	lines = [ fp.readline() for fp in fps ] #read first lines of each alignment files

	lengths = [ len(line[21:-1]) for line in lines ] #save lengths for empty segments
	sequence = ''.join( [line[21:-1] for line in lines ] )
	header = title[:20]
	print >>to_fp, '%-20s %s'% (header, sequence)

	while 1 :
		#read a line from each file
		lines = [ fp.readline() for fp in fps ]
		#remove empty lines
		filtered_lines = filter( len,  lines )

		#if all lines are empty
		#stop the loop
		if not filtered_lines :
			break

		#check each segment if they are none or not, if none fill the gaps (fill_gaps function)
		#and join all of the segments.
		sequence = ''.join( [ fill_gaps(line[21:-1], length) for line, length in zip(lines,lengths) ] )
		header = filtered_lines[0][:20] #get a line header
	
		print >>to_fp, '%-20s %s'% (header, sequence)
			
	[ fp.close() for fp in fps ]

	return to_fp


def select_largest_iteration( blast_fp ) :
	'''
	This function returns largest iteration start byte position in the blast result file
	with iteration number
	'''
        fp = blast_fp
	fp.seek(0)

        blast_iteration_starts = []

        l = fp.readline()
        while l :
                if l[:len(psiblast_start)] == psiblast_start :
                        blast_iteration_starts.append( fp.tell() )
		elif l[:len(psiblast_end_of_result_marker)] == psiblast_end_of_result_marker :
			blast_iteration_starts.append( fp.tell()-len(l) )
			break

                l = fp.readline()
        else :
                blast_iteration_starts.append( fp.tell() )


	blast_iteration_sizes = []
	for i, j in zip( blast_iteration_starts[:-1], blast_iteration_starts[1:]) :
		blast_iteration_sizes.append( j-i )

        #print fn, blast_iteration_sizes

	#just get the maximum stuff!!
        for size, idx in reversed( sorted( zip(blast_iteration_sizes, range( len(blast_iteration_sizes) )) ) ) :
                if size < 200 :
			print "Warning! PSI-BLAST iteration has no results.", idx+1
                break

	if len(blast_iteration_starts) == 1 :
		return 0, 0  #start from 0 for blast result

        if idx != len(blast_iteration_sizes)-1 :
                print >>sys.stderr, 'Warning! the last ieration is not biggest.', idx, len(blast_iteration_sizes) -1, blast_iteration_sizes

	return blast_iteration_starts[idx], idx+1



def select_last_iteration( blast_fp ) :
	'''
	This function returns largest iteration start byte position in the blast result file
	with iteration number
	'''
        fp = blast_fp
	fp.seek(0)

        blast_iteration_starts = []

        l = fp.readline()
        while l :
                if l[:len(psiblast_start)] == psiblast_start :
                        blast_iteration_starts.append( fp.tell() )
		elif l[:len(psiblast_end_of_result_marker)] == psiblast_end_of_result_marker :
			blast_iteration_starts.append( fp.tell()-len(l) )
			break

                l = fp.readline()
        else :
                blast_iteration_starts.append( fp.tell() )

	blast_iteration_sizes = []
	for i, j in zip( blast_iteration_starts[:-1], blast_iteration_starts[1:]) :
		blast_iteration_sizes.append( j-i )

        #print blast_iteration_sizes
	#print blast_iteration_starts

        for size, idx in reversed( zip(blast_iteration_sizes, range( len(blast_iteration_sizes) ) ) ) :
                if size < 200 :
			print "Warning! PSI-BLAST iteration has no results.", idx+1
			continue
                break

	if len(blast_iteration_starts) == 1 :
		return 0, 0  #start from 0 for blast result

        if idx != len(blast_iteration_sizes)-1 :
                print >>sys.stderr, 'Warning! the last ieration is not biggest.', idx, len(blast_iteration_sizes) -1, blast_iteration_sizes

	return blast_iteration_starts[idx], idx+1




######################3
# Protocols
######################
def build_psiblast_from_fragments( input_fn, output_fn, fragment_definitions ) :
        '''
        This function run psiblast for fragments and combine the MSA for each fragments into one.
        Note that the combination will be as in the order in the fragment definition list.
        fragment_definition: [ (0, 100), (101, 120), ...]
        Also note that the fragment definitions are in the way of python slice start and end.. 
        '''

        gi, annotation, seq = read_fasta_file(input_fn)
        input_seq = seq

        name_base = input_fn.split('/')[-1]

        fragment_name_base = name_base +'_frag_%d.fa'
        input_name_base = name_base + '_frag_%d.in'

        aln_list = []

        for fragnum, (start, end) in enumerate( fragment_definitions ) :
                frag_fa = fragment_name_base % fragnum
                seq = input_seq[start:end]
                write_fasta_file( frag_fa, seq, name = annotation + ' ' +str(fragnum) )

                blastrunner = PsiblastRunner( frag_fa )
                #debugging for fast speed
                #blastrunner.iteration = 1
                #debugging end
                result_fn = blastrunner.run_psiblast_original()

                blastmsa = BlastMSA( frag_fa, result_fn )
                blastmsa.parse()
                input_name = input_name_base%fragnum
                blastmsa.build_psiblast_alignment_input( open(input_name, 'w') )
                aln_list.append( input_name )

        str_fp = combine_psiblast_input_files_horizontally( aln_list, title=annotation )

        fp = open( output_fn, 'w' )
        fp.write( str_fp.getvalue() )
        fp.close()

			

if __name__ == "__main__" :
	import sys
	#fp = open(sys.argv[1])
	#blast_msa = BlastMSA()
	#blast_msa.parse(fp)

	#fp = open( sys.argv[2] )
	#fp.readline()
	#seq = ''.join( [ s.strip() for s in fp.readlines() ] )
	#blast_msa.set_query_seq( seq )

	#print blast_msa.to_string()
	#blast_msa.build_psiblast_alignment_input(sys.stdout)
	
	psiblast = PsiblastRunner( sys.argv[1] )
	psiblast.cpu = 4
	psiblast.run_psiblast()
	#psiblast_result_fn = psiblast.run_initial_blast()

	#blast_msa = BlastMSA( sys.argv[1], psiblast_result_fn )
	#blast_msa.parse()
	
	#blast_msa.build_psiblast_alignment_input( sys.stdout )
